/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

                   /* SET TABS TO 4 TO READ THESE SOURCES */


#include "xdipo.h"

FD_xdipo *fdui;


#define DISEQC_DELAY	15


void set_ts_filt(int fd, uint16_t pid, dmx_pes_type_t pestype)
{
struct dmx_pes_filter_params pesFilterParams;

if(debug_flag)
	{
	fprintf(stderr, "set_ts_filt(): arg fd=%d pid=%d pestype=%d\n", fd, pid, pestype);
	}

pesFilterParams.pid     = pid;
pesFilterParams.input   = DMX_IN_FRONTEND;
pesFilterParams.output  = DMX_OUT_TS_TAP;
pesFilterParams.pes_type = pestype;
pesFilterParams.flags   = DMX_IMMEDIATE_START;

if(ioctl(fd, DMX_SET_PES_FILTER, &pesFilterParams) < 0) 
	{
	fprintf(stderr, "set_ts_filt(): DMX_SET_PES_FILTER failed for pid=%d: ", pid);
	perror("reason ");
	}

} /* end function set_ts_filt */


static inline void msleep(uint32_t msec)
{
struct timespec req = { msec / 1000, 1000000 * (msec % 1000) };

while (nanosleep(&req, &req) ) ;
} /* end function msleep */


int send_diseqc_command(int *data, int length)
{
int a, i;
struct dvb_diseqc_master_cmd *cmd;
int voltage;
int tone_flag, highband_flag;
struct dvb_frontend_parameters feparams;
//struct dvb_frontend_info fe_info;
unsigned long freq, srate;
char temp[80];
char temp2[20];

if(debug_flag)
	{
	fprintf(stderr,\
	"send_diseqc_command(): arg data=%p length=%d using fd_frontend=%d\n",\
	data, length, fd_frontend);

	fprintf(stderr, "sending: ");	
	for(i = 0; i < length; i++) fprintf(stderr, "%02x ", data[i]);
	fprintf(stderr, "\n");
	}

if(length > 6)
	{
	fprintf(stderr, "send_diseqc_command(): length %d longer then 6, error.\n", length);

	return 0;
	}

#define SLOF (11700*1000UL)
#define LOF1 (9750*1000UL)
#define LOF2 (10600*1000UL)

/* if pol is vertical then voltage is 13V, if horizontal then 18V */
/* if band is low then tone is off, else tone is on */

freq = frequency * 1000UL;
srate = symbolrate *1000UL;

if(debug_flag)
	{
	fprintf(stderr, "freq=%lu\n", freq);

	fprintf(stderr, "setting frontend:\n\
	freq=%lu polarization=%c inversion=%d srate=%lu fec_inner=%d modulation=%d\n",\
	freq, polarization, INVERSION_OFF, srate, FEC_AUTO, QAM_64);
	}

if(freq > SLOF)
	{
	freq -= LOF2;
	highband_flag = 1;
	}
else
	{
	freq -= LOF1;
	highband_flag = 0;
	}

feparams.frequency = freq;
feparams.inversion = INVERSION_OFF;
feparams.u.qam.symbol_rate = srate; 
feparams.u.qpsk.fec_inner = FEC_AUTO;
feparams.u.qam.modulation = QAM_64;


if( toupper(polarization) == 'V') voltage = SEC_VOLTAGE_13; else voltage = SEC_VOLTAGE_18; 
if( highband_flag) tone_flag = SEC_TONE_ON; else tone_flag = SEC_TONE_OFF;

/* if a diseqc command requested */
if(length)
	{
	/* no tone, we are going to send pulses */
	a = ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): set tone failed.\n");
		perror("reason ");
		return 0;
		}

	/* Mosteck SG 2100 A only moves with 13 V?? */
	a = ioctl(fd_frontend, FE_SET_VOLTAGE, SEC_VOLTAGE_13);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): set voltage failed.\n");
		perror("reason ");
		return 0;
		}

	msleep(15);

//struct dvb_diseqc_master_cmd
//{
//uint8_t msg [6];        /*  { framing, address, command, data [3] } */
//uint8_t msg_len;        /*  valid values are 3...6  */
//};

	cmd = malloc(sizeof(struct dvb_diseqc_master_cmd) );
	if(! cmd)
		{
		fprintf(stderr, "send_diseqc_command(): malloc struct dvb_diseqc_master_cmd failed.\n");

		return 0;
		}

	if(debug_flag)
		{
		fprintf(stderr, "send_diseqc_command(): cmd=%p\n", cmd);
		}

	sprintf(temp, "f=%.2f p=%c s=%d command=", frequency, polarization, symbolrate);

	for(i = 0; i < length; i++)	
		{
		cmd -> msg[i] = (int8_t) data[i];

		sprintf(temp2, "%02x ", cmd -> msg[i]);

		strcat(temp, temp2);	
		} /* end for all bytes in diseqc command */

	if(gui_flag)
		{
		fl_set_object_label(fdui -> main_form_command_output_field, temp);
		}

	cmd -> msg_len = length;

	a = ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, cmd);
	free(cmd);
	if(a)
		{
		fprintf(stderr, "send_diseqc_command(): send master command failed.\n");
		perror("reason ");
		return 0;
		}

	usleep(200000);
	} /* end if length (any diseqc command) */

/* set the required voltage for polarization */
a = ioctl(fd_frontend, FE_SET_VOLTAGE, voltage);
if(a)
	{
	fprintf(stderr, "send_diseqc_command(): set voltage failed.\n");
	perror("reason ");
	return 0;
	}

/* restore the 22 kHz tone if it was requested for high band */
a = ioctl(fd_frontend, FE_SET_TONE, tone_flag);
if(a)
	{
	fprintf(stderr, "send_diseqc_command(): set tone failed.\n");
	perror("reason ");
	return 0;
	}

a = ioctl(fd_frontend, FE_SET_FRONTEND, &feparams);
if(a < 0)
	{
	fprintf(stderr, "send_diseqc_command(): set frontend failed.\n");
	perror("reason ");	
	return 0;
	}

return 1;
} /* end function send_diseqc_command */


int get_frontend_status()
{
int a;
fe_status_t fe_status;

if(debug_flag)
	{
	fprintf(stderr, "get_frontend_status(): arg none, using fd_frontend=%d\n", fd_frontend);
	}

a = ioctl(fd_frontend, FE_READ_STATUS, &fe_status);
if(a)
    {
    fprintf(stderr, "test_lock():  FE_GET_STATUS failed.\n");
    perror("reason ");

    return 0;
    }

if(fe_status & FE_HAS_SIGNAL) carrier_flag = 1;
else carrier_flag = 0;

if(fe_status & FE_HAS_VITERBI) viterbi_flag = 1;
else viterbi_flag = 0;

if(fe_status & FE_HAS_SYNC) sync_flag = 1;
else sync_flag = 0;

if(fe_status & FE_HAS_LOCK) lock_flag = 1;
else lock_flag = 0;

if(debug_flag)
	{
	fprintf(stderr,\
	"get_status() carrier_flag=%d viterbi_flag=%d sync_flag=%d lock_flag=%d\n",\
	carrier_flag, viterbi_flag, sync_flag, lock_flag);
	}

if(gui_flag)
	{
	if(carrier_flag)
		{
		fl_set_object_color(fdui -> main_form_carrier_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_carrier_output_field, FL_RED, FL_GREEN);
		}

	if(viterbi_flag)
		{
		fl_set_object_color(fdui -> main_form_viterbi_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_viterbi_output_field, FL_RED, FL_GREEN);
		}

	if(sync_flag)
		{
		fl_set_object_color(fdui -> main_form_sync_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_sync_output_field, FL_RED, FL_GREEN);
		}

	if(lock_flag)
		{
		fl_set_object_color(fdui -> main_form_lock_output_field, FL_GREEN, FL_RED);
		}
	else
		{
		fl_set_object_color(fdui -> main_form_lock_output_field, FL_RED, FL_GREEN);
		}

	} /* end if gui_flag */

return 1;
} /* end function get_frontend_status */

