/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


                                  /* SET TABS TO 4 TO READ THIS SOURCE */

#include "xdipo.h"

extern FD_xdipo *fdui;
extern FD_xdipo *fd_main_form;

FL_APPEVENT_CB optimize_idle_cb;

static int old_commands_form_drive_west_button_state = -1;
static int old_commands_form_drive_east_button_state = -1;
static int temp_west[] = {0xe0, 0x31, 0x69, 0x00};
static int temp_east[] = {0xe0, 0x31, 0x68, 0x00};
static int temp_halt[] = {0xe0, 0x31, 0x60};
static struct timeval timeout;
static char buffer[DVR_BUFFER_SIZE];
static int signal_strength, signal_to_noise_ratio, bit_error_rate;
//static int uncorrected_blocks;


int idle_cb(XEvent *ev, void *data)
{
int a, b;
time_t now;
double da;
char temp[120];

/* update bitrate indicator */
now = time(NULL);
if(now >= old_time + 1)
	{
	if(! (recorder_mode & RECORDER_RECORD_MODE) )
		{
		total_seconds = (double)now;
		total_bytes_read = 0.0;
		total_bitrate = 0.0;
		}
	else
		{
		da = total_bytes_read / ( (double)now - total_seconds);
		da *= 8.0;		// bps
		da /= 1000.0;	// kbps
	
		total_bitrate = da;
		}

	/* total bitrate indicator */
	sprintf(temp, "%.0f", total_bitrate);
	fl_set_object_label(fdui -> main_form_bitrate_output_field, temp);

	old_time = now;
	}

process_timers();

a = fl_get_button(fdui -> installer_commands_form_start_motor_driving_west_button);
if(old_commands_form_drive_west_button_state != a)
	{
	if(a)
		{
		if(old_commands_form_drive_west_button_state == 0) send_diseqc_command(temp_west, 4);
		}
	else
		{
		if(old_commands_form_drive_west_button_state == 1) send_diseqc_command(temp_halt, 3);
		}
	old_commands_form_drive_west_button_state = a;
	}

a = fl_get_button(fdui -> installer_commands_form_start_motor_driving_east_button);
if(old_commands_form_drive_east_button_state != a)
	{
	if(a)
		{
		if(old_commands_form_drive_east_button_state == 0) send_diseqc_command(temp_east, 4);
		}
	else
		{
		if(old_commands_form_drive_east_button_state == 1) send_diseqc_command(temp_halt, 3);
		}
	old_commands_form_drive_east_button_state = a;
	}

/* if output requested */
if(recorder_mode & RECORDER_RECORD_MODE)
	{
	/* Initialize the set of active sockets. */
	FD_ZERO (&active_fd_set);

	/* listen to dvr device */
	FD_SET(fd_dvr, &active_fd_set);     

	/* wait until input arrives on one or more active sockets. */
	read_fd_set = active_fd_set;

	timeout.tv_sec = 0;
	timeout.tv_usec = 100000000;
	a = select(FD_SETSIZE, &read_fd_set, NULL, NULL, &timeout);
	if(a < 0)
		{
		fprintf(stderr, "xdipo: idle_cb(): select failed, aborting.\n");
		perror ("reason ");

		exit(1);
		}

	if(FD_ISSET (fd_dvr, &read_fd_set) )
		{
		a = read(fd_dvr, buffer, DVR_BUFFER_SIZE);
		if(a == -1)
			{
			fprintf(stderr, "xdipo: idle_cb(): read fd_dvr failed\n");
			perror("reason ");

			if(errno == EOVERFLOW)
				{
				user_warning_dvr_buffer_size(gui_flag);

				fl_set_object_color(fdui -> main_form_output_filename_output_field, FL_DARKORANGE, FL_COL1);
				}
			else
				{
				close_output_files();

				recorder_mode = RECORDER_STOP_MODE;

				set_record_button(0);

				enable_controls(1);
			
				return 0;
				}
			}

//		fprintf(stderr, "read %d bytes\n", a);

		total_bytes_read += (float)a;

		b = fwrite(buffer, sizeof(char), a, ofptr);
		if(b != a)
			{
			fprintf(stderr, "xdipo: idle_cb(): fwrite failed\n");
			perror("reason ");

			if(errno == ENOSPC)
				{
				fl_set_object_label(fdui -> main_form_output_filename_output_field, "DISK FULL");
				}
			else
				{
				fl_set_object_label(fdui -> main_form_output_filename_output_field, "ERROR");
				}
		
			close_output_files();

			recorder_mode = RECORDER_STOP_MODE;

			set_record_button(0);

			enable_controls(1);
			
			return 0;
			}

//		usleep(1000);
		} /* end if FD_ISSET */

	} /* end if recorder_mode & RECORDER_RECORD_MODE */

print_status();

return 1;
} /* end function idle_cb */


int print_status()
{
int a;
double da, db;
char temp[256];
int beep_range;
int beep_frequency;
static int count = 0; // -Wall
double fbe;
double bit_errors;
double deebee;
double signal;
time_t now;

int megabytes;
int percentage;

/* save processor cycles, ioctl is expensive */
if(! beep_mode)
	{ 	
	count++;
	if(count % 10) return 1;
	}

/* get front end info */
/*
bit error rate, multiple of 10-9,
2500 = 2.5 10-6 is 1 error in 400000
*/
a = ioctl(fd_frontend, FE_READ_BER, &bit_error_rate);
if(a < 0)
	{
	fprintf(stderr, "xdipo: print_status(): error=%d reading fd_frontend bit_error_rate\n", a);
	perror("reason ");
	}

if(debug_flag)
	{
//	fprintf(stderr, "bit_error_rate=%d\n", bit_error_rate);
	}

fbe = (double) bit_error_rate * 10e-9;
bit_errors = 1.0 / fbe;
if(debug_flag)
	{
//	fprintf(stderr, "BIT ERRORS=1 in %.0f fbe=%.3f\n", bit_errors, fbe);
	}

sprintf(temp, "%d", bit_error_rate);
fl_set_object_label(fdui -> main_form_bit_error_rate_text_field, temp);

if(bit_error_rate > 20000)
	{
	fl_set_object_color(fdui -> main_form_bit_error_rate_text_field,\
	FL_RED, FL_RED);
	}
else if(bit_error_rate > 5000)
	{
	fl_set_object_color(fdui -> main_form_bit_error_rate_text_field,\
	FL_DARKORANGE, FL_DARKORANGE);
	}
else if(bit_error_rate != 0)
	{
	fl_set_object_color(fdui -> main_form_bit_error_rate_text_field,\
	FL_YELLOW, FL_YELLOW);
	}
else
	{
	fl_set_object_color(fdui -> main_form_bit_error_rate_text_field,\
	FL_GREEN, FL_GREEN);
	}

if(recorder_mode & RECORDER_RECORD_MODE)
	{
	if(bit_error_rate != 0)
		{
		if(debug_flag)
			{
//			fprintf(stderr, "fpbe=%p fileno(fpbe)=%d\n", fpbe, fileno(fpbe) );
			}

		fprintf(fpbe, "%s bit_errors=%d\n", get_universal_time_as_string(), bit_error_rate);
		fflush(fpbe);
		}
	}

/* uncorrected blocks, seems always zero? */
/*
a = ioctl(fd_frontend, FE_READ_UNCORRECTED_BLOCKS, &uncorrected_blocks);
if(a < 0)
	{
	fprintf(stderr, "xdipo: print_status(): error=%d reading fd_frontend uncorrected blocks\n", a);
	perror("reason ");
	}
fprintf(stderr, "WAS uncorrected_blocks=%d\n", uncorrected_blocks);
*/

/* signal to noise */
a = ioctl(fd_frontend, FE_READ_SNR, &signal_to_noise_ratio);
if(a < 0)
	{
	fprintf(stderr, "xdipo: print_status(): error=%d reading fd_frontend SNR\n", a);
	perror("reason ");
	}

if(debug_flag)
	{
//	fprintf(stderr, "signal_to_noise_ratio=%d \n", signal_to_noise_ratio);
	}

/* signal strength */
a = ioctl(fd_frontend, FE_READ_SIGNAL_STRENGTH, &signal_strength);
if(a < 0)
	{
	fprintf(stderr, "xdipo: print_status(): error=%d reading fd_frontend signal_strenth\n", a);
	perror("reason ");
	}

if(debug_flag)
	{
//	fprintf(stderr, "signal_strength=%d \n", signal_strength);
	}

/* normalize */
signal = (double)signal_strength / 65535.0;

/* make range 0-1 */
deebee = (double)signal_to_noise_ratio / 65535.0;

/* set signal strength indicator */
fl_set_slider_value(fdui -> main_form_signal_strength_slider, signal);

/* set signal to noise indicator */
fl_set_dial_value(fdui -> main_form_signal_strength_indicator, deebee);

/* limit range of indicator and beep */
/* sometimes this seems to get negative */
if(deebee < 0.0) deebee = 0.0;

/* make range 0-20 */
deebee *= MAX_SNR_SCALE;

/* print maximum anyways */
sprintf(temp, "%.0f", deebee);
fl_set_object_label(fdui -> main_form_signal_strength_text_field, temp);

beep_range = BEEP_MAXIMUM_FREQ - BEEP_MINIMUM_FREQ;

/* create a beep sound, higher frequency is more signal */
da = 0; /* -Wall */
if(beep_mode)
	{
	da = deebee / MAX_SNR_SCALE;
	db = da * beep_range;
	beep_frequency = BEEP_MINIMUM_FREQ + db;

	if(debug_flag)
		{
//		fprintf(stderr, "BEEP FREQ=%d\n", beep_frequency);
		}

	beep_freq(beep_frequency);
	}

/* disk space indicator */
if(get_free_disk_space(output_dir, &megabytes, &percentage) )
	{
	/* set slider indicating position in file */
	da = percentage / 100.0;
	fl_set_slider_value(fdui -> main_form_seek_slider, da);

	if(recorder_mode & RECORDER_RECORD_MODE)
		{
		now = time(NULL);

		sprintf(temp,\
		"%d minutes  disk usage %d%%\n%dMB free",\
		(int)(now - recording_start_time) / 60,\
		percentage, megabytes);
		}
	else
		{
		sprintf(temp, "disk usage %d%%\n%dMB free",\
		percentage, megabytes);
		}

	fl_set_object_label(fdui -> main_form_seek_slider, temp);
	}
else
	{
	if(debug_flag)
		{
//		fprintf(stderr, "get_free_disk_space(): failed\n");
		}
	}

if(! get_frontend_status() )
	{
	return 0;
	}

return 1;
} /* end function print_status */


