/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


                                  /* SET TABS TO 4 TO READ THIS SOURCE */


#include "xdipo.h"

FD_xdipo *fdui;


struct program *lookup_program(char *name)
{
struct program *pa;

for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return pa;
	}

return 0; /* not found */
} /* end function lookup_program */


struct program *install_program_at_end_of_list(char *name)
{
struct program *plast, *pnew;

if(debug_flag)
	{
	fprintf(stderr,\
	"install_program_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_program(name);
if(pnew)
	{
	/* free previous definition */
	return pnew; /* already there */
	}

/* create new structure */
pnew = (struct program *) calloc(1, sizeof(*pnew) );
if(! pnew)
	{
	fprintf(stderr,\
	"install_program_at_end_of_list(): could not allocate space for struct\n");

	return 0;
	}

pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = programtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !programtab[0] ) programtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
programtab[1] = pnew;

return pnew; /* pointer to new structure */
}/* end function install_program */


int delete_program(char *name)/* delete entry from double linked list */
{
struct program *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stderr, "delete_program(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = programtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify programtab[0] */
	if(pprev == 0) programtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify programtab[1] */
	if(pnext == 0) programtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */
}/* end function delete_program */


int delete_all_programs()/* delete all entries from table */
{
struct program *pa;

if(debug_flag)
	{
	fprintf(stderr, "delete_all_programs() arg none\n");
	}

while(1)
	{	
	pa = programtab[0];
	if(! pa) break;
	programtab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa); /* free structure */
	}/* end while all structures */

programtab[1] = 0;
return 1;
}/* end function delete_all_programs */


int save_programs()
{
char temp[4096];
struct program *pa;
FILE *fptr;
FILE *pptr;

if(debug_flag)
	{
	fprintf(stderr, "save_programs (): arg none\n");
	}

/* save the new file as a tmp */
sprintf(temp, "%s/.xdipo/streams.dat.tmp", home_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"save_programs (): could not open file %s for write\n",\
	temp);
	return 0;
	}

for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(fptr,\
	"%d %.2f %c %d %.2f %s %.2f %s\n",\
	pa -> satellite, pa -> frequency,  pa -> polarization, pa -> symbolrate, pa -> angle,\
	pa -> satellite_name, pa -> gw_angle, pa -> pids);
	}	

fclose(fptr);

/* remove any old .bak file */
sprintf(temp, "%s/.xdipo/streams.dat.bak", home_dir);
if(unlink(temp) != 0)
	{
	fprintf(stderr, "save_programs(): could not unlink %s\n", temp);

//	return 0;
	}

/* rename the old file to .bak */
sprintf(temp, "mv %s/.xdipo/streams.dat %s/.xdipo/streams.dat.bak",\
home_dir, home_dir);
pptr = popen(temp, "r");
pclose(pptr);

/* rename the temp file to the corrrect name */
sprintf(temp, "mv %s/.xdipo/streams.dat.tmp %s/.xdipo/streams.dat",\
home_dir, home_dir);
pptr = popen(temp, "r");
pclose(pptr);

return 1;
}/* end function save_programs */


int load_programs()
{
int a;
FILE *fptr;
char temp[READSIZE];
char temp2[READSIZE];
struct program *pa;
extern char *strsave();
double frequency;
int polarization;
int satellite;
int symbolrate;
double angle;
struct stat *statptr;
int file_size;
int line;
int transport_stream_id;
int service_type;
char pola;
char satellite_name[40];
double gw_angle;
char pids[80];

if(debug_flag)
	{
	fprintf(stderr, "load_programs() arg none\n");
	}
	
delete_all_programs();

sprintf(temp, "%s/.xdipo/streams.dat", home_dir); 
fptr = fopen(temp, "r");
if(! fptr)
	{
	fl_show_alert("Could not open file", temp, "for read", 0);

	if(debug_flag)
		{
		fprintf(stderr,\
		"load_programs(): could not open file %s for read\n",\
		temp);
		}

	return 0;
	} 

/*
need the length of this file, to prevent generating zero output length
files from defective (zero length) streams.dat
*/
/* allocate space for structure stat */
statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr)
	{
	fprintf(stderr,\
	"load_programs(): could not alllocate space for struct stat\n");

	return 0;
	}

/* fill the structure with data for this file */
fstat(fileno(fptr), statptr);

/* get filesize */
file_size = statptr -> st_size;

/* free the structure */
free(statptr);

/* test the filesize */
if(file_size == 0)
	{
	fprintf(stderr,\
	"load_programs(): %s has zero length aborting\n", temp);

	return 0;
	}

/* read the file */
line = 0;
while(1)
	{
	a = readline(fptr, temp);/* closes file on EOF */
	if(a == EOF) break;/* empty file or last line */

	line++;

	/* for compatibility with older versions */

	transport_stream_id = 0;
	service_type = 0;

	/* support old version 0.1.1 */
	angle = 0.0;
	satellite_name[0] = 0;
	pids[0] = 0;

	a = sscanf(temp, "%d %lf %c %d %lf %s %lf %s",\
	&satellite, &frequency, &pola, &symbolrate, &angle, satellite_name, &gw_angle, pids);
	polarization = pola;

	/* old versions did not fill in pids, set for full stream */
	if( strlen(pids) == 0 ) strcpy(pids, "8192");

//fprintf(stderr, "WAS pids=%s\n", pids);

	sprintf(temp2, "%d %.2f %c %d %.2f %s %.2f %s",\
	satellite, frequency, polarization, symbolrate, angle, satellite_name, gw_angle, pids);

	if(a >= 4)
		{
		pa = install_program_at_end_of_list(temp2);
		if(! pa)
			{
			if(debug_flag)
				{
				fprintf(stderr,\
				"load_programs(): cannot install entry %s\n", temp);

				return 0;
				}
			} /* end if install failed */

		pa -> frequency = frequency;
		pa -> polarization = polarization;
		pa -> satellite = satellite;
		pa -> symbolrate = symbolrate;
		pa -> angle = angle;
		pa -> gw_angle = gw_angle;
		strncpy(pa -> satellite_name, satellite_name, 39);
		strncpy(pa -> pids, pids, 79);
		}/* end if correct number of arguments */
	else
		{
		sprintf(temp, "line %d", line);
		fl_show_alert(\
		"File format error streams.dat", temp, "aborting", 0); 

//		exit(1);
		}
	}/* end while all lines in streams.dat */

sort_program_list();

return 1;
}/* end function load_programs */


int add_program_in_position()
{
int i;
struct program *pa;
char temp[READSIZE];
int satellite;
double frequency;
char polarization;
int symbolrate;
char *ptr;
int a;
double angle;
char direction[10];
char satellite_name[40];
double gw_angle;
char pids[80];

if(debug_flag)
	{
	fprintf(stderr,\
	"add_program_in_position(): arg none\n");
	}

ptr = (char *)fl_get_input(fdui -> main_form_satellite_number_input_field);
if(! ptr) return 0;
satellite = atoi(ptr);

ptr = (char *)fl_get_input(fdui -> main_form_frequency_input_field);
if(! ptr) return 0;
frequency = atof(ptr);

ptr = (char *)fl_get_input(fdui -> main_form_polarization_input_field);
if(! ptr) return 0;
polarization = ptr[0];

ptr = (char *)fl_get_input(fdui -> main_form_symbolrate_input_field);
if(! ptr) return 0;
symbolrate = atoi(ptr);

ptr = (char *)fl_get_input(fdui -> main_form_angle_input_field);
if(! ptr) return 0;
a = sscanf(ptr, "%lf %1[EW]s", &angle, direction);
if( toupper(direction[0]) == 'E') angle *= -1.0;

ptr = (char *)fl_get_input(fdui -> main_form_satellite_name_input_field);
if(! ptr) return 0;
strncpy(satellite_name, ptr, 39);

ptr = (char *)fl_get_input(fdui -> main_form_satellite_location_input_field);
if(! ptr) return 0;
a = sscanf(ptr, "%lf %1[EW]s", &gw_angle, direction);
if( toupper(direction[0]) == 'E') gw_angle *= -1.0;

ptr = (char *)fl_get_input(fdui -> main_form_pids_input_field);
if(! ptr) return 0;

/* replace any spaces in pids with '_' */
for(i = 0; i < 79; i++)
	{
	pids[i] = ptr[i];

	if(pids[i] == 0) break;
	if(pids[i] == ' ') pids[i] = '_';
	}

sprintf(temp, "%d %.2f %c %d %.2f %s %.2f %s",\
satellite, frequency, polarization, symbolrate, angle, satellite_name, gw_angle, pids);

/* sets status to new entry if found not to exist */
pa = install_program_at_end_of_list(temp);
if(! pa)
	{
	if(debug_flag)
		{
		fprintf(stderr,\
		"add_program_in_position(): cannot install entry %s\n", temp);

		return 0;
		}
	} /* end if install failed */

pa -> satellite = satellite;
pa -> frequency = frequency;
pa -> polarization = polarization;
pa -> symbolrate = symbolrate;
pa -> angle = angle;
pa -> gw_angle = gw_angle;
strncpy(pa -> satellite_name, satellite_name, 39);
strncpy(pa -> pids, pids, 79);
pa -> status = 0;

/* sort the list, cause if item < first entry was entered it would be at the end. */
sort_program_list();

return 1;
}/* end function add_program_in_position */


int swap_program_position(struct program *ptop, struct program *pbottom)
{
struct program *punder;
struct program *pabove;

if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or programtab[0]) must now point to ptop */
if(! punder)
	{
	programtab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in programtab */
if(! pabove)
	{
	programtab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_program_position */


int sort_program_list()
/*
sorts the double linked program list.
doing some sort of bubble sort.
*/
{
struct program *pa;
struct program *pb;
int swap_flag;

if(debug_flag)
	{
	fprintf(stderr, "sort_program_list(): arg none\n");
	}

/* sort the double linked list */
while(1)/* go through program again and again */
	{
	swap_flag = 0;
	for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(debug_flag)
				{
				fprintf(stderr, "pa->name=%s pb->name=%s\n",\
				pa -> name, pb -> name);
				}

			if(pa -> gw_angle < pb -> gw_angle)
				{
				swap_flag = swap_program_position(pa, pb);
				}
			else if(pa -> gw_angle == pb -> gw_angle)
				{
				if(pa -> frequency < pb -> frequency)
					{
					swap_flag = swap_program_position(pa, pb);
					}
				}

			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, program is sorted */
	if(! swap_flag) break;
	}/* end while go through program again and again */

return 1;
}/* end function sort_program_list */


int show_programs(int position)
{
int a, b, i;
char temp[1024];
struct program *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int first_new_program_line;
int find_first_new_program_flag;
int maxline;
char pids[80];

if(debug_flag)
	{
	fprintf(stderr, "show_programs(): arg position=%d using selected_stream=%d\n",\
	position, selected_stream);
	}

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> main_form_stream_browser);

fl_freeze_form(fdui -> main_form);

fl_clear_browser(fdui -> main_form_stream_browser);

first_new_program_line = -1; /* keep gcc -Wall from complaining */
find_first_new_program_flag = 1;
browser_line = 1;

for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(position == SHOW_UNREAD_AT_TOP)
		{
		/* want to show the first unaccessed at top of display */
		if(find_first_new_program_flag)
			{
			if(pa -> status == 1)
				{
				first_new_program_line = browser_line;
				find_first_new_program_flag = 0;
				}
			}
		}
	
	strcpy(formatstr, "@b@i@f");
//	strcpy(formatstr, "");

	if(pa -> status == 0) strcat(formatstr, "@C0");/* black */
	
	if(pa -> angle < 0) a = 'E';
	else a = 'W'; 

	if(pa -> gw_angle < 0) b = 'E';
	else b = 'W';

	/* replace any '_' in pids with spaces */
	i = 0;
	while(1)
		{
		pids[i] = pa -> pids[i];

		if(pids[i] == 0) break;
		if(pids[i] == '_') pids[i] = ' ';

		i++;
		}

	sprintf(temp,\
	"%s %s:%.2f %c:%.2f %c:%.2f:%c:%d:%d:%s",\
	formatstr, pa -> satellite_name, fabs(pa -> gw_angle), b, fabs(pa -> angle), a,\
	pa -> frequency, pa -> polarization, pa -> symbolrate, pa -> satellite, pids);

	temp[1023] = 0;
	fl_add_browser_line(fdui -> main_form_stream_browser, temp);

	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_stream_browser, first_new_program_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last program visible */
	maxline = fl_get_browser_maxline(fdui -> main_form_stream_browser);
	fl_set_browser_topline(fdui -> main_form_stream_browser, maxline);
	}

if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_stream_browser, 1);
	}

if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> main_form_stream_browser, browser_topline);
	}	

fl_unfreeze_form(fdui -> main_form);

/*
maintain visible indication that line is selected with single click
*/
fl_select_browser_line(fdui -> main_form_stream_browser, selected_stream );

if(position == SHOW_SELECTED_IN_MIDDLE)
	{
	fl_set_browser_topline(fdui -> main_form_stream_browser, selected_stream - 10);
	}	
else
	{
	/* show the entry */
	fl_set_browser_topline(fdui -> main_form_stream_browser, selected_stream);
	}

return 1;
} /* end function show_programs */


char *get_selected_program_definition(int line)
{
struct program *pa;

if(debug_flag)
	{
	printf(\
	"get_selected_program_definition(): arg line=%d\n",\
	line);
	}

for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line) return pa -> satellite_name;
	}

return 0; /* not found */
} /* end function get_selected_program_definition */


int satellite_name_to_browser_line(char *name, int *line)
{
struct program *pa;

if(debug_flag)
	{
	fprintf(stderr, "satellite_name_to_browser_line(): arg name=%s\n", name);
	}

if(! name) return 0;

for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> satellite_name, name) == 0)
		{
		*line = pa -> browser_line;
		return 1;
		}
	}

return 0;
} /* end function satellite_name_to_browser_line */



int search_and_select_satellite()
{
struct program *pa;
char *keyword;
int case_sensitive_flag;
int found;
static int last_found_line;

if(debug_flag)
	{
	fprintf(stdout, "search_and_select_satellite(): arg none\n");
	}

keyword = (char *)fl_get_input(fdui -> main_form_search_input_field);
if(! keyword) return 0;

case_sensitive_flag = fl_get_button(fdui -> main_form_case_sensitive_button);

found = 0;
/* pa points to next entry */
for(pa = programtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* search from last position (set to zero if keyword changes */
	if(pa -> browser_line <= last_found_line) continue;
	
	if(search_in_for(pa -> name, keyword, case_sensitive_flag) )
		{
		found = 1;
		}

	if(found)
		{
//		free(selected_text);
//		selected_text = strsave(pa -> name);

		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_stream_browser, pa -> browser_line);

		/* show the entry */
		fl_set_browser_topline(fdui -> main_form_stream_browser, pa -> browser_line);

//		/* select the entry */
//		fl_set_input(fdui -> main_form_text_input_field, pa -> name);

		/* remember where we are */
		last_found_line = pa -> browser_line;
		
		selected_program = atoi(pa -> name);

//		set_program_input(selected_start_program);

		/* return found */
		return 1;
		}/* end if found */
	}/* end for all program's */

if(last_found_line) fl_show_alert(keyword, "end of list", "", 0);
else fl_show_alert(keyword, "not found", "", 0);

last_found_line = 0;

/* return not found */
return 0;
}/* end function search_and_select_satellite */



int search_in_for(char *database, char *data, int case_sensitive)
{
/* This function returns 1 if all words in data are present in database */
char *ptra, *ptrb, *ptrc;
int match_failed_flag;
int last_string_flag;
char *strsave();
char *strstr_case_insensitve();

if(debug_flag)
	{
	fprintf(stdout,\
	"search_in_for(): arg database=%s data=%s case_sensitive=%d\n",\
	database, data, case_sensitive);
	}

/* argument check */
if(! database) return 0;
if(! data) return 0;
if( (case_sensitive != 0) && (case_sensitive != 1) ) return 0;

/*
Test if some data,
data may consist of one or more words separated by spaces
*/
if(! data[0]) return 0;/* no data */

/* test if data present in database */
/* for all entries in data */
ptra = data;
ptrb = strsave(data);/* create space */
match_failed_flag = 0;
last_string_flag = 0;
while(1)
	{
	ptrc = ptrb;/* point to start availeble space */
	while(1)/* string in data to ptrb */
		{
		*ptrc = *ptra;
		if(! *ptra) break;/* end data */
		if(*ptra == ' ')/* end of string in data */
			{
			*ptrc = 0;
			ptra++;/* scip the space */
			break;
			}
		ptra++;
		ptrc++;
		}/* end while all characters in string */
	/* test if string present in database */
	if(case_sensitive)
		{
		if(! strstr(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}
	else
		{
		if(! strstr_case_insensitive(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}/* end if not case sensitive */
	if(! *ptra) break;/* end filter */
	}/* end while all strings in filter */	
free(ptrb);/* release space */

if(match_failed_flag) return 0;
return 1;
} /* end function search_in_for */


char *strstr_case_insensitive(char *s, char *t)
{
int i, j, k;
for(i = 0; s[i] != '\0'; i++)
	{
	for(j = i, k = 0; t[k] != '\0' &&  toupper(s[j]) == toupper(t[k]);\
	j++, k++)
	;
	if(t[k] == '\0') return s + i;
	}
return 0;
}/* end function strstr_case_insensitive */



