/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


                                  /* SET TABS TO 4 TO READ THIS SOURCE */


#include "xdipo.h"

FD_xdipo *fdui;


int calculate_angle_and_elevation(\
	double longitude, double latitude, double satellite_position,\
	double *calculated_angle, double *elevation)
{
double H;
double R;
double ALT;
double B, LO, LA, SAT, AZI, ELE;

if(debug_flag)
	{
	fprintf(stderr,\
	"calculate_angle_and_elevation(): arg longitude=%.2f latitude=%.2f satellite_position=%.2f\n",\
	longitude, latitude, satellite_position); 
	}

/* rad degrees conversion */
H = 180.0 / M_PI;

/* circumference earth (40000 km) / (2 * PI) */
R = 6371.0;

/* guess, depends on sat */
//ALT = 35822.0;
ALT = 38696.47;

LO = longitude;

//LA = latitude;
LA = fabs(latitude);

SAT = satellite_position;

B = LO - SAT;

AZI = 180.0 + (H * atan2( tan(B / H), sin(LA / H) ) );

if(debug_flag)
	{
	fprintf(stderr, "calculate_angle_and_elevation(): initial AZI=%.2f\n", AZI);
	}

if(AZI < 180.0) AZI = -1.0 * (180.0 - AZI);
else AZI = -1.0 * (AZI - 180.0);

//AZI += 0.5;

*calculated_angle = AZI;

ELE = \
H * atan2(\
( cos(LA / H) * cos(B / H) ) - (R / (R + ALT) ), \
sqrt( 1 - (cos(LA / H) * cos(LA / H) * cos(B / H) * cos(B / H) ) ) \
);

if(debug_flag)
	{
	fprintf(stderr, "calculate_angle_and_elevation(): initial ELE=%.2f\n", ELE);
	}

if(ELE < 1) return 0; /* satellite below horizon */

//ELE += 0.5;

*elevation = ELE;

if(debug_flag)
	{
	fprintf(stderr,\
	"calculate_angle_and_elevation(): calculated_angle=%.2f elevation=%.2f\n",\
	*calculated_angle, *elevation);
	}

return 1;
} /* end function calculate_angle_and_elevation */


