/* 
xdipo

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xdipo.h"

FD_xdipo *fdui;

FILE *filefd;


int save_setting(char *type, char *string)
{
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xdipo/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "w");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for write:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stderr, "\nCannot open setting file %s for write\n",\
		pathfilename);
		}
	return 0;
	}

fprintf(setting_file, "%s", string);
fclose(setting_file);
return 1;	
}/* end function save_setting */


char *load_setting(char *type)
{
int a;
char temp[1024];
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xdipo/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "r");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for read:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stderr, "\nCannot open setting file %s for read\n",\
		pathfilename);
		}
	return 0;
	}

a = readline(setting_file, temp);/* closes file if EOF */	
if(a != EOF) fclose(setting_file);
return strsave(temp);
}/* end function load_setting */


int save_fontsizes()
{
char temp[TEMP_SIZE];
sprintf(temp,\
"%d %d",\
main_form_fontsize,\
timer_form_fontsize\
);

if(! save_setting("fontsizes", temp) ) return 0;

return 1;
}/* end function save_fontsizes */


int load_fontsizes()
{
char *data_read;
 
data_read = (char *) load_setting("fontsizes");
if(! data_read) return 0;
sscanf(data_read,\
"%d %d",\
&main_form_fontsize,\
&timer_form_fontsize\
);

free(data_read);

/* for main form */
fl_set_browser_fontsize(fdui -> main_form_stream_browser, integer_to_fontsize(main_form_fontsize) );

/* for timer form */
fl_set_browser_fontsize(fdui -> timer_form_browser, integer_to_fontsize(timer_form_fontsize) );

return 1;
}/* end function load_fontsizes */


char *expand_tabs(char *text, int tabsize)
{
int c, i, j, k;
int tabs;
char *space;

if(debug_flag)
	{
	fprintf(stderr, "expand_tabs(): arg text=%s tabsize=%d\n",\
	text, tabsize);
	}

/* argument check */
if(! text) return 0;
if(tabsize < 0) return 0;

/* determine how many tabs, so we can calculate the required space */
i = 0;
tabs = 0;
while(1)
	{
	c = text[i];
	if(c == 0) break;
	if(c == 9) tabs++;
	i++;
	}/* end while all characters in text */

space = malloc(strlen(text) + (tabs * tabsize) - tabs + 1 + 1);
if(! space) return 0;

/* copy and expand */
i = 0;
j = 0;
while(1)
	{
	c = text[i];
	if(c == 9)
		{
		for(k = 0; k < tabsize; k++)
			{
			space[j] = ' ';
			j++;
			}
		i++;
		continue;
		}
	else
		{
		space[j] = c;
		}
	if(c == 0) break;
	i++;
	j++;
	}
return space;
}/* end function expand_tabs */



int save_search_settings()
{
char temp[1024];

if(debug_flag)
	{
	fprintf(stderr, "save_search_settings()\n");
	}

sprintf(temp, "%d %s", fl_get_button(fdui -> main_form_case_sensitive_button), fl_get_input(fdui -> main_form_search_input_field) );
if(! save_setting("search_settings", temp) ) return 0;

return 1;
} /* end function save_search_settings */



int load_search_settings()
{
char *data_read;
int search_case;
char *ptr;

if(debug_flag)
	{
	fprintf(stderr, "load_search_settings()\n");
	}
 
data_read = (char *) load_setting("search_settings");
if(! data_read) return 0;
 
sscanf(data_read, "%d", &search_case);
fl_set_button(fdui -> main_form_case_sensitive_button, search_case);

ptr = strstr(data_read, " ") + 1;
fl_set_input(fdui -> main_form_search_input_field, ptr);

free(data_read);

return 1;
} /* end function load_search_settings */

