/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


                                  /* SET TABS TO 4 TO READ THIS SOURCE */


#include "xdipo.h"

FD_xdipo *fdui;

FILE *filefd;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc( strlen(s) +  1);
if(! p) return 0;

strcpy(p, s);

return p;
} /* end function strsave */


int readline(FILE *file, char *contents)
{
int a, c, i;

if(debug_flag)
	{
	fprintf (stderr, "readline(): arg file=%lu\n", (long)file);
	}

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}
	if(c == '\n')
		{
		contents[i] = 0;/* string termination */
		return 1;/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */
/*fclose(file);*/
if(debug_flag)
	{
	fprintf(stderr,\
	"readline(): line to long, returning 0 contents=%s\n", contents);
	}
return 0;
}/* end function readline */


int integer_to_fontsize(int fontsize)
{
int size;

size = 0;/* only for clean compilation with -Wall */

/* tiny snall normal medium large huge */
if(fontsize == 0) size = FL_TINY_SIZE;
if(fontsize == 1) size = FL_SMALL_SIZE;
if(fontsize == 2) size = FL_NORMAL_SIZE;
if(fontsize == 3) size = FL_MEDIUM_SIZE;
if(fontsize == 4) size = FL_LARGE_SIZE;
if(fontsize == 5) size = FL_HUGE_SIZE;

return(size);
}/* end function integer to fontsize */


int ogettime(
char *weekday,
char *month,
int *monthday,
int *hour,
int *minute,
int *second,
int *year
)
{
time_t I;
char *datestring;

time(&I); /*sets unsigned long int I to seconds since start 1970*/
datestring = ctime(&I); /*char *   converts I to character string date and time*/

sscanf(datestring,"%s %s %d %d:%d:%d %d",\
weekday,month,monthday,hour,minute,second,year);/*no &, already pointers*/

return 1;
}/* end function ogettime */


char *get_universal_time_as_string()
{
time_t now;
static char temp[80];
size_t strftime();
struct tm *universal_time, *gmtime();
char *ptr;
char *strsave();
char *old_locale, *saved_locale;

if(debug_flag)
	{
	fprintf(stderr, "get_universal_time_as_string(): arg none\n");
	}

/* 
set the locale to english language even if env shows otherwise
*/
/* Get the name of the current locale */
old_locale = setlocale (LC_ALL, NULL);
     
/* Copy the name so it won't be clobbered by `setlocale' */
saved_locale = strdup (old_locale);
if(saved_locale == NULL)
	{
	fprintf(stderr, "get_universal_time_as_string(): Out of memory\n");

	return 0;
	}     

/* 
Now change the locale to US english for 'Wed and not for example 'wo'
server may not be able to parse something like 'wo'.
*/
ptr = setlocale(LC_ALL, "en_US");
if(! ptr)
	{
	fprintf(stderr, "get_universal_time_as_string(); could not setlocale\n");

	return 0;
	}

/* 
decided to adapt to universal time (UT == GMT).
New format:
from rfc1036.html:
Wdy, DD Mon YY HH:MM:SS TIMEZONE 
*/

now = time(0);
universal_time = gmtime(&now);
strftime (temp, 511, "%a, %d %b %Y %H:%M:%S GMT", universal_time);

ptr = strsave(temp);

if(debug_flag)
	{
	fprintf(stderr, "get_universal_time_as_string(): ptr=%s\n", ptr);
	}

/* Restore the original locale */
setlocale (LC_ALL, saved_locale);
free (saved_locale);

return(ptr);
}/* end function get_universal_time_as_string */


int get_free_disk_space(char *dir, int *megabytes, int *percentage)
{
int a, found;
FILE *fptr;
FILE *pptr;
char temp[TEMP_SIZE];
char dummy[TEMP_SIZE];
int usage;
int available;

if(debug_flag)
	{
//	fprintf(stderr, "get_free_disk_space(): arg dir=%s\n", dir);
	}

/* parameter check */
if(! dir) return 0;

/* extract the mount point of the first part of the directory name to */

/* execute df */
sprintf(temp, "df > %s/.xdipo/disk_space", home_dir);
pptr = popen(temp, "w");
pclose(pptr);

/* format of the file */
/*
Filesystem           1k-blocks      Used Available Use% Mounted on
/dev/hda7              9076364   7125700   1489604  83% /
/dev/hda5                23302      3681     18418  17% /boot
shmfs                   495768         0    495768   0% /dev/shm
/dev/hdb5              6135204   4768980   1054568  82% /redhat
/dev/hdd1             28842748  22280728   5096896  82% /video
/dev/hda1             29288192  25004352   4283840  86% /big
/dev/hdd2              9775248   7474388   2300860  77% /reiser
*/

/* read and parse the disk_space_file */
sprintf(temp, "%s/.xdipo/disk_space", home_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr, "get_free_disk_space(): could not open %s for read\n", temp);
	}

/*
first line reads:
Filesystem           1k-blocks      Used Available Use% Mounted on
skip it.
*/
fscanf(fptr, "%[^\n]", temp);

found = 0;
while(1)
	{
	/* read a line */
	a = fscanf(fptr, "%s %s %s %d %d%% %s",\
	dummy, dummy, dummy, &available, &usage, temp);	

	if(debug_flag)
		{
/*
		fprintf(stderr, "a = %d dummy=%s available=%d usage=%d temp=%s\n",\
		a, dummy, available, usage, temp);
*/
		}

	if(a == EOF) break;

	if(a != 6) continue;

	if(strcmp(temp, dir) == 0) /* '/' versus  '/' */
		{
		found = 1;	

		break;
		}

	if(strcmp(temp , "/") == 0) continue; /* everything else is related to '/', so we skip that */

	if(strstr(dir, temp) == dir) /* '/reiser' is in '/reiser/video/in' */
		{
		found = 1;

		break;
		}

	} /* end while read lines */

fclose(fptr);

if(! found) return 0;

*megabytes = available / 1024;
*percentage = usage; 

return 1;
} /* end function get_free_disk_space */


int parse_pids_string(char *string)
{
int i;
const char delimiters[] = " ";
char *token, *running;
int pid[MAX_PIDS];
int pids;

if(debug_flag)
	{
	fprintf(stderr, "parse_pids_string(): arg string=%s\n", string);
	}

running = strsave(string);
if(! running)
	{
	fprintf(stderr, "parse_pids_string(): strsave(string) failed\n");

	return 0;
	}

use_filter_flag = 0;
pids = 0;
while(1)
	{
	token = strsep(&running, delimiters);
	if(! token) break;

	/* test if looking for service id */
	if( use_filter_flag)
		{
		pid[0] = 8192;

		pids = 1;

		service_id = atoi(token); /* the second field */

		break;
		}

	/* test for service id indicator 'p' */
	if(strcmp(token, "s") == 0) /* the first field */
		{
		use_filter_flag = 1;

		/* get second filed with service id */
		continue;
		}

	pid[pids] = atoi(token);

	pids++;
	} /* end while all pids in string */

free(running);

if(debug_flag)
	{
	fprintf(stderr, "parse_pids_string(): %d pids found\n", pids);
	}

if(pids > MAX_PIDS)
	{
	fprintf(stderr,\
	"xdipo: parse_pids_string(): %d pids, max allowed is %d, ignoring extra.\n",\
	pids, MAX_PIDS);

	pids = MAX_PIDS;
	}

/* close any open demux devices */
for(i = 0; i < pids_open; i++)
	{
	close(fd_demux[i]);
	}

pids_open = 0;
for(i = 0; i < pids; i++)
	{	
	fd_demux[i] = open(demux_device_name, O_RDWR);
	if(fd_demux[i] < 0)
		{
		fprintf(stderr, "xdipo: main(): open demux device %s failed, aborting.\n", demux_device_name);
		perror("reason ");
		exit(1);
		}

	pids_open++;

	if(debug_flag)
		{
		fprintf(stderr, "parse_pids_string(): setting pid=%d\n", pid[i]);
		}

	set_ts_filt(fd_demux[i], pid[i], DMX_PES_OTHER);
	} /* end for pids */

/* no token found is error */
return 1;
} /* end function parse_pids_string */


int get_this_time(\
time_t this_time,
char *weekday,
char *month,
int *monthday,
int *hour,
int *minute,
int *second,
int *year
)
{
char *datestring;

/* char *, converts time_t I to character string date and time */
datestring = ctime(&this_time);

sscanf(datestring, "%s %s %d %d:%d:%d %d",\
weekday, month, monthday, hour, minute, second,year);/*no &, already pointers*/

return 1;
}/* end function get_this_time */


int weekday_to_digit(char *weekday_str)
{
int day = -1; // -Wall

/*
The abbreviations for the days of week in ctime() are:
`Sun', `Mon', `Tue', `Wed', `Thu', `Fri', `Sat'.
*/
if(strcmp(weekday_str, "Sun") == 0) day = 0;
if(strcmp(weekday_str, "Mon") == 0) day = 1;
if(strcmp(weekday_str, "Tue") == 0) day = 2;
if(strcmp(weekday_str, "Wed") == 0) day = 3;
if(strcmp(weekday_str, "Thu") == 0) day = 4;
if(strcmp(weekday_str, "Fri") == 0) day = 5;
if(strcmp(weekday_str, "Sat") == 0) day = 6;

return day;
} /* end function weekday_to_digit */


int month_to_digit(char *month_str)
{
int month = 0; // -Wall

/*
The abbreviations for the months in ctime() are:
`Jan', `Feb', `Mar', `Apr', `May', `Jun',
`Jul', `Aug', `Sep', `Oct', `Nov', `Dec'.
*/

if(strcmp(month_str,"Jan") == 0) month = 1; 
if(strcmp(month_str,"Feb") == 0) month = 2;
if(strcmp(month_str,"Mar") == 0) month = 3; 
if(strcmp(month_str,"Apr") == 0) month = 4; 
if(strcmp(month_str,"May") == 0) month = 5; 
if(strcmp(month_str,"Jun") == 0) month = 6; 
if(strcmp(month_str,"Jul") == 0) month = 7; 
if(strcmp(month_str,"Aug") == 0) month = 8; 
if(strcmp(month_str,"Sep") == 0) month = 9; 
if(strcmp(month_str,"Oct") == 0) month = 10;
if(strcmp(month_str,"Nov") == 0) month = 11;
if(strcmp(month_str,"Dec") == 0) month = 12;

return month;
} /* end function month_to_digit */


int enable_controls(int mode)
{
if(debug_flag)
	{
	printf("enable_controls(): mode=%d\n", mode);
	}

if(mode == 0)
	{
//	fl_deactivate_object(fdui -> main_form_title);

	fl_deactivate_object(fdui -> main_form_recorder_menu);
	fl_deactivate_object(fdui -> main_form_frequency_input_field);
	fl_deactivate_object(fdui -> main_form_polarization_input_field);
	fl_deactivate_object(fdui -> main_form_symbolrate_input_field);
	fl_deactivate_object(fdui -> main_form_stream_browser);
	fl_deactivate_object(fdui -> main_form_add_stream_button);
	fl_deactivate_object(fdui -> main_form_delete_stream_button);
	fl_deactivate_object(fdui -> main_form_nudge_motor_west_button);
	fl_deactivate_object(fdui -> main_form_nudge_motor_east_button);
	fl_deactivate_object(fdui -> main_form_nudge_tilt_motor_up_button);
	fl_deactivate_object(fdui -> main_form_nudge_tilt_motor_down_button);

//	fl_deactivate_object(fdui -> main_form_east_text);
//	fl_deactivate_object(fdui -> main_form_west_text);

	fl_deactivate_object(fdui -> main_form_increase_nudge_size_button);
	fl_deactivate_object(fdui -> main_form_decrease_nudge_size_button);
	fl_deactivate_object(fdui -> main_form_toggle_step_timed_mode_button);
	fl_deactivate_object(fdui -> main_form_toggle_goto_angle_sattelite_button);
	fl_deactivate_object(fdui -> main_form_store_present_satellite_number_button);
	fl_deactivate_object(fdui -> main_form_goto_button);
	fl_deactivate_object(fdui -> main_form_increment_satellite_number_button);
	fl_deactivate_object(fdui -> main_form_decrement_satellite_number_button);
	fl_deactivate_object(fdui -> main_form_satellite_number_input_field);
	fl_deactivate_object(fdui -> main_form_angle_input_field);
	fl_deactivate_object(fdui -> main_form_angle_down_button);
	fl_deactivate_object(fdui -> main_form_angle_up_button);
	fl_deactivate_object(fdui -> main_form_step_value_input_field);
	fl_deactivate_object(fdui -> main_form_signal_strength_indicator);

//	fl_deactivate_object(fdui -> main_form_signal_strength_text_field);
//	fl_deactivate_object(fdui -> main_form_signal_strength_text_text_field);

//	fl_deactivate_object(fdui -> main_form_bit_error_rate_text_text_field);
//	fl_deactivate_object(fdui -> main_form_bit_error_rate_text_field);

	fl_deactivate_object(fdui -> main_form_beep_button);
	fl_deactivate_object(fdui -> main_form_signal_strength_slider);
	fl_deactivate_object(fdui -> main_form_installer_button);
	fl_deactivate_object(fdui -> main_form_elevation_output_field);
	fl_deactivate_object(fdui -> main_form_elevation_output_field_text);
	fl_deactivate_object(fdui -> main_form_satellite_name_input_field);
	fl_deactivate_object(fdui -> main_form_satellite_location_input_field);
	fl_deactivate_object(fdui -> main_form_longitude_input_field);
	fl_deactivate_object(fdui -> main_form_latitude_input_field); 

//	fl_deactivate_object(fdui -> main_form_local_clock);

//	fl_deactivate_object(fdui -> main_form_command_output_field);
//	fl_deactivate_object(fdui -> main_form_command_output_field_text);

//	fl_deactivate_object(fdui -> main_form_output_filename_output_field);
//	fl_deactivate_object(fdui -> main_form_output_filename_output_field_text);

	fl_deactivate_object(fdui -> main_form_pids_input_field);

	fl_deactivate_object(fdui -> main_form_seek_slider);

//	fl_deactivate_object(fdui -> main_form_record_button);

	fl_deactivate_object(fdui -> main_form_exit_switch);
	
	fl_deactivate_object(fdui -> main_form_search_input_field);
	fl_deactivate_object(fdui -> main_form_case_sensitive_button);
	fl_deactivate_object(fdui -> main_form_search_button);

//	fl_deactivate_object(fdui -> main_form_fontsize_up_button);
//	fl_deactivate_object(fdui -> main_form_fontsize_down_button);

//	fl_deactivate_object(fdui -> installer_commands_form_form_title);
	fl_deactivate_object(fdui -> installer_commands_form_recalculate_positions_button);
	fl_deactivate_object(fdui -> installer_commands_form_satellite_number_input_field);
	fl_deactivate_object(fdui -> installer_commands_form_recalculate_param1_input_field);
	fl_deactivate_object(fdui -> installer_commands_form_recalculate_param2_input_field);
	fl_deactivate_object(fdui -> installer_commands_form_start_motor_driving_west_button);
	fl_deactivate_object(fdui -> installer_commands_form_halt_motor_button);
	fl_deactivate_object(fdui -> installer_commands_form_start_motor_driving_east_button);
	fl_deactivate_object(fdui -> installer_commands_form_set_west_limit_button);
	fl_deactivate_object(fdui -> installer_commands_form_disable_limits_button);
	fl_deactivate_object(fdui -> installer_commands_form_set_east_limit_button);
	fl_deactivate_object(fdui -> installer_commands_form_goto_reference_position_button);
	fl_deactivate_object(fdui -> installer_commands_form_enable_limits_button);
	fl_deactivate_object(fdui -> installer_commands_form_close_button);
	
	} /* end if mode is 0 */
else if(mode == 1)
	{
//	fl_activate_object(fdui -> main_form_title);

	fl_activate_object(fdui -> main_form_recorder_menu);
	fl_activate_object(fdui -> main_form_frequency_input_field);
	fl_activate_object(fdui -> main_form_polarization_input_field);
	fl_activate_object(fdui -> main_form_symbolrate_input_field);
	fl_activate_object(fdui -> main_form_stream_browser);
	fl_activate_object(fdui -> main_form_add_stream_button);
	fl_activate_object(fdui -> main_form_delete_stream_button);
	fl_activate_object(fdui -> main_form_nudge_motor_west_button);
	fl_activate_object(fdui -> main_form_nudge_motor_east_button);
	fl_activate_object(fdui -> main_form_nudge_tilt_motor_up_button);
	fl_activate_object(fdui -> main_form_nudge_tilt_motor_down_button);

//	fl_activate_object(fdui -> main_form_east_text);
//	fl_activate_object(fdui -> main_form_west_text);

	fl_activate_object(fdui -> main_form_increase_nudge_size_button);
	fl_activate_object(fdui -> main_form_decrease_nudge_size_button);
	fl_activate_object(fdui -> main_form_toggle_step_timed_mode_button);
	fl_activate_object(fdui -> main_form_toggle_goto_angle_sattelite_button);
	fl_activate_object(fdui -> main_form_store_present_satellite_number_button);
	fl_activate_object(fdui -> main_form_goto_button);
	fl_activate_object(fdui -> main_form_increment_satellite_number_button);
	fl_activate_object(fdui -> main_form_decrement_satellite_number_button);
	fl_activate_object(fdui -> main_form_satellite_number_input_field);
	fl_activate_object(fdui -> main_form_angle_input_field);
	fl_activate_object(fdui -> main_form_angle_down_button);
	fl_activate_object(fdui -> main_form_angle_up_button);
	fl_activate_object(fdui -> main_form_step_value_input_field);

	fl_activate_object(fdui -> main_form_signal_strength_indicator);

//	fl_activate_object(fdui -> main_form_signal_strength_text_field);
//	fl_activate_object(fdui -> main_form_signal_strength_text_text_field);

//	fl_activate_object(fdui -> main_form_bit_error_rate_text_text_field);
//	fl_activate_object(fdui -> main_form_bit_error_rate_text_field);

	fl_activate_object(fdui -> main_form_beep_button);
	fl_activate_object(fdui -> main_form_signal_strength_slider);
	fl_activate_object(fdui -> main_form_installer_button);

//	fl_activate_object(fdui -> main_form_elevation_output_field);
//	fl_activate_object(fdui -> main_form_elevation_output_field_text);

	fl_activate_object(fdui -> main_form_satellite_name_input_field);
	fl_activate_object(fdui -> main_form_satellite_location_input_field);
	fl_activate_object(fdui -> main_form_longitude_input_field);
	fl_activate_object(fdui -> main_form_latitude_input_field); 

//	fl_activate_object(fdui -> main_form_local_clock);

//	fl_activate_object(fdui -> main_form_command_output_field);
//	fl_activate_object(fdui -> main_form_command_output_field_text);

//	fl_activate_object(fdui -> main_form_output_filename_output_field);
//	fl_activate_object(fdui -> main_form_output_filename_output_field_text);

	fl_activate_object(fdui -> main_form_pids_input_field);
	fl_activate_object(fdui -> main_form_seek_slider);
	fl_activate_object(fdui -> main_form_record_button);
	fl_activate_object(fdui -> main_form_exit_switch);

	fl_activate_object(fdui -> main_form_search_input_field);
	fl_activate_object(fdui -> main_form_case_sensitive_button);
	fl_activate_object(fdui -> main_form_search_button);

//	fl_activate_object(fdui -> main_form_fontsize_up_button);
//	fl_activate_object(fdui -> main_form_fontsize_down_button);

//	fl_activate_object(fdui -> installer_commands_form_form_title);

	fl_activate_object(fdui -> installer_commands_form_recalculate_positions_button);
	fl_activate_object(fdui -> installer_commands_form_satellite_number_input_field);
	fl_activate_object(fdui -> installer_commands_form_recalculate_param1_input_field);
	fl_activate_object(fdui -> installer_commands_form_recalculate_param2_input_field);
	fl_activate_object(fdui -> installer_commands_form_start_motor_driving_west_button);
	fl_activate_object(fdui -> installer_commands_form_halt_motor_button);
	fl_activate_object(fdui -> installer_commands_form_start_motor_driving_east_button);
	fl_activate_object(fdui -> installer_commands_form_set_west_limit_button);
	fl_activate_object(fdui -> installer_commands_form_disable_limits_button);
	fl_activate_object(fdui -> installer_commands_form_set_east_limit_button);
	fl_activate_object(fdui -> installer_commands_form_goto_reference_position_button);
	fl_activate_object(fdui -> installer_commands_form_enable_limits_button);
	fl_activate_object(fdui -> installer_commands_form_close_button);

	} /* end if mode is 1 */
	
return 1;
} /* end function enable_controls */

	
int user_warning_dvr_buffer_size(int gui_flag)
{
if(debug_flag)
	{
	fprintf(stderr,\
	"user_warning_dvr_buffer_size(): arg giu_flag=%d\n", gui_flag);
	}

if(gui_flag)
	{
	fl_show_alert(\
	"DVB driver dvr buffer overflow, your recording now contains errors and may not play back!",\
	"Modify dmxdev.h in DVB driver so it reads:",\
	"#define DVR_BUFFER_SIZE (100*188*1024) // (original 10*188*1024)\nthen reload module dvbcode.o",\
	0);
	}
else
	{ 
	fprintf(stderr,\
	"DVB driver dvr buffer overflow.\n\
Modify dmxdev.h in the DVB driver so that it reads:\n\
#define DVR_BUFFER_SIZE (100*188*1024) // (original 10*188*1024),\n\
and then reload the DVB modules.\n");
	}	

fprintf(stderr, "WARNING: your recording now contains errors and may not play back!\n");

return 1;
} /* end function user_warning_dvr_buffer_size */

	
int open_output_files()
{
char temp[4096];

if(debug_flag)
	{
	fprintf(stderr,\
	"open_output_files(): arg none, using output_filename=%s dvr_device_name=%s\n",\
	output_filename, dvr_device_name);
	}

if(! use_filter_flag)
	{
	ofptr = fopen(output_filename, "w");
	if(! ofptr)
		{
		fprintf(stderr,\
		"xdipo: open_output_files(): could not open file %s for write.\n", output_filename);
		perror("reason ");

		return 0;
		}	
	}
else
	{
	sprintf(temp, "%s -p %d 2>/dev/zero > %s", FILTER, service_id, output_filename);
	ofptr = popen(temp, "w");	

	if(debug_flag)
		{
		fprintf(stderr, "executing: %s\n", temp);
		}

	if(! ofptr)
		{
		fprintf(stderr,\
		"xdipo: open_output_files(): could not execute %s.\n", temp);
		perror("reason ");

		return 0;
		}
	
	}

sprintf(temp, "%s.bit_errors", output_filename);
if(bit_errors_log_filename) free(bit_errors_log_filename);
bit_errors_log_filename = strsave(temp);
if(! bit_errors_log_filename)
	{
	fprintf(stderr,\
	"xdipo: open_output_files(): could not allocate space for bit_errors_log_filename, aborting.\n");

	exit(1);
	}

fpbe = fopen(bit_errors_log_filename, "w");
if(! fpbe)
	{
	fprintf(stderr,\
	"xdipo: open_output_files(): could not open file %s for write.\n", bit_errors_log_filename);
	perror("reason ");

//	fpbe = 0;
	close_output_files();

	return 0;
	}

fd_dvr = open(dvr_device_name, O_RDONLY | O_NONBLOCK);
if(fd_dvr < 0)
	{
	fprintf(stderr,\
	"xdipo: process_timers(): could not open dvr device %s for read.\n",\
	dvr_device_name);

	perror("reason ");

//	fd_dvr = 0;
	close_output_files();

	return 0;
	}

return 1;
} /* end function open_output_files */


int close_output_files()
{
int i;
char temp[80];

if(debug_flag)
	{
	fprintf(stderr,\
	"close_output_files(): arg none, using ofptr=%p fpbe=%p fd_dvr=%d pids_open=%d\n",\
	ofptr, fpbe, fd_dvr, pids_open);
	
	for(i = 0; i < pids_open; i++)
		{
		fprintf(stderr, "close_output_files(): fd_demux[%d]=%d\n", i, fd_demux[i]);
		}
	}

/* close any open demux devices */
for(i = 0; i < pids_open; i++)   
	{
	if( close(fd_demux[i]) != 0)
		{
		sprintf(temp, "xdipo: close_ouput_files(): close(fd_demux[%d] failed because ", i);

		perror(temp);
		}

	fd_demux[i] = 0;
	}
pids_open = 0;

if(ofptr)
	{
	if(! use_filter_flag)
		{
		if( fclose(ofptr) != 0)
			{
			sprintf(temp, "xdipo: close_ouput_files(): fclose(ofptr) failed because ");

			perror(temp);
			}
		}
	else
		{
		if( pclose(ofptr) == -1)
			{
			/* returns status value of child process, as in system, see libc.info */

			sprintf(temp, "xdipo: close_ouput_files(): pclose(ofptr) failed because ");

			perror(temp);
			}	
		}

	ofptr = 0;
	}
if(fpbe)
	{
	if( fclose(fpbe) != 0)
		{
		sprintf(temp, "xdipo: close_ouput_files(): fclose(fpbe) failed because ");

		perror(temp);
		}	

	fpbe = 0;
	}

if(fd_dvr > 0)
	{
	if( close(fd_dvr) != 0)
		{
		sprintf(temp, "xdipo: close_ouput_files(): close(fd_dvr) failed because ");

		perror(temp);
		}

	fd_dvr = 0;
	}

return 1;
} /* end function close_output_files */

 

